
local AssignSoldier = ...

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	AssignSoldier:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		Mover = { DataType = "BUILDING_PART_MOVER" },
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 0 }
		    }
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0 }
	    }
	})
end

function registerBasicLadder(_ladderNumber, _resourceNeeded)
-- Apply building asset processor
AssignSoldier:registerAssetProcessor("models/basic/basicLadder" .. _ladderNumber .. ".fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register Main prefab nodes
AssignSoldier:registerAssetId("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_0_Ladder" .. _ladderNumber .. "_RootPart", "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_ROOT_PREFAB")
AssignSoldier:registerAssetId("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_1_Ladder" .. _ladderNumber .. "_Tiling1Part", "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_TILING1_PREFAB")
AssignSoldier:registerAssetId("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_2_Ladder" .. _ladderNumber .. "_Tiling2Part", "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_TILING2_PREFAB")
AssignSoldier:registerAssetId("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_3_Ladder" .. _ladderNumber .. "_TopPart", "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_TOP_PREFAB")
--------------------------------------------------------------------------

local defaultNodePrefixList = {
	"ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_TOP",
	"ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_TILING1",
	"ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_TILING2"
}

-- Register simple building part assets
for i, nodePrefix in ipairs(defaultNodePrefixList) do
	registerDefaultBuildingPart(nodePrefix)
end

-- Register the stone gate set small squared tower root part
AssignSoldier:register({
	DataType = "BUILDING_PART",
	Id = "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_ROOT_PART",
	Name = "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_ROOT_NAME",
	Description = "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_ROOT_DESC",
	Category = "EXTENSION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_ROOT_PREFAB",
		EndPart = "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_TOP_PART",
		FillerList = {
			"ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_TILING2_PART"
                },
		BasementFillerList = {
			"ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_TILING1_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = true
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = _resourceNeeded, Quantity = 1 }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 0.2 }
	},
	AssetBuildingFunction = "ASSIGN_SOLDIER_FUNCTION_LADDER" .. _ladderNumber
})
AssignSoldier:register({
		DataType = "BUILDING_FUNCTION_WORKPLACE",
                Id = "ASSIGN_SOLDIER_FUNCTION_LADDER" .. _ladderNumber,
		WorkerCapacity = 1,
                RelatedJob = { Job = "SOLDIER", Behavior = "WORK_BEHAVIOR" }
})

--------------------------------------------------------------------------

-- Register flag assigner component on the collider's type to 'PLATFORM'

AssignSoldier:configurePrefabFlagList("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_3_Ladder" .. _ladderNumber .. "_TopPart", { "PLATFORM" })
AssignSoldier:configurePrefabFlagList("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_2_Ladder" .. _ladderNumber .. "_Tiling2Part", { "PLATFORM" })
AssignSoldier:configurePrefabFlagList("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_1_Ladder" .. _ladderNumber .. "_Tiling1Part", { "PLATFORM" })
 
-- Register grass clearing components

--AssignSoldier:registerPrefabComponent("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_0_Ladder" .. _ladderNumber .. "_RootPart", {
--	DataType = "COMP_DIRT_RECTANGLE",
--	Size = { 4, 4 }
--})

-- Override building part components properties (generated by the building asset processor) to add building zones
AssignSoldier:registerPrefabComponent("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_0_Ladder" .. _ladderNumber .. "_RootPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 0, 0 }
})
end

registerBasicLadder("1", "STONE")
registerBasicLadder("2", "STONE")
registerBasicLadder("3", "STONE")
registerBasicLadder("4", "STONE")
registerBasicLadder("6", "STONE")
registerBasicLadder("7", "STONE")
registerBasicLadder("8", "STONE")
registerBasicLadder("9", "STONE")
registerBasicLadder("10", "STONE")
registerBasicLadder("11", "STONE")
registerBasicLadder("12", "WOOD")

function registerSpecialLadder(_ladderNumberBase, _ladderNumber,_resourceNeeded)
-- Apply building asset processor
AssignSoldier:registerAssetProcessor("models/basic/basicLadder" .. _ladderNumber .. ".fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register Main prefab nodes
AssignSoldier:registerAssetId("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_0_Ladder" .. _ladderNumber .. "_RootPart", "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_ROOT_PREFAB")
AssignSoldier:registerAssetId("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_2_Ladder" .. _ladderNumber .. "_Tiling2Part", "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_TILING2_PREFAB")
AssignSoldier:registerAssetId("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_2_Ladder" .. _ladderNumber .. "_Tiling3Part", "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_TILING3_PREFAB")
AssignSoldier:registerAssetId("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_3_Ladder" .. _ladderNumber .. "_TopPart", "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_TOP_PREFAB")
--------------------------------------------------------------------------

local defaultNodePrefixList = {
	"ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_TOP",
	"ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_TILING2",
	"ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_TILING3"
}

-- Register simple building part assets
for i, nodePrefix in ipairs(defaultNodePrefixList) do
	registerDefaultBuildingPart(nodePrefix)
end

-- Register the additional root part
AssignSoldier:register({
	DataType = "BUILDING_PART",
	Id = "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_ROOT_PART",
	Name = "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_ROOT_NAME",
--	Description = "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_ROOT_DESC",
--	Category = "EXTENSION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_ROOT_PREFAB",
		EndPart = "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_TOP_PART",
		FillerList = {
			"ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_TILING3_PART",
			"ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_TILING2_PART"
                },
		MinimumScale = 0,
		BasementScale = 0,
		IsVertical = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = _resourceNeeded, Quantity = 1 }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 0.2 }
	},
	AssetBuildingFunction = "ASSIGN_SOLDIER_FUNCTION_LADDER" .. _ladderNumber
})
AssignSoldier:register({
		DataType = "BUILDING_FUNCTION_WORKPLACE",
                Id = "ASSIGN_SOLDIER_FUNCTION_LADDER" .. _ladderNumber,
		WorkerCapacity = 1,
                RelatedJob = { Job = "SOLDIER", Behavior = "WORK_BEHAVIOR" }
})

--------------------------------------------------------------------------

-- Register flag assigner component on the collider's type to 'PLATFORM'

AssignSoldier:configurePrefabFlagList("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_3_Ladder" .. _ladderNumber .. "_TopPart", { "PLATFORM" })
AssignSoldier:configurePrefabFlagList("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_2_Ladder" .. _ladderNumber .. "_Tiling2Part", { "PLATFORM" })
AssignSoldier:configurePrefabFlagList("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_2_Ladder" .. _ladderNumber .. "_Tiling3Part", { "PLATFORM" })

-- Override building part components properties (generated by the building asset processor) to add building zones
AssignSoldier:registerPrefabComponent("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_0_Ladder" .. _ladderNumber .. "_RootPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 0, 0 }
})

-- Register Main prefab nodes
AssignSoldier:registerAssetId("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_0_Ladder" .. _ladderNumberBase .. "_RootPart", "ASSIGN_SOLDIER_LADDER" .. _ladderNumberBase .. "_ROOT_PREFAB")
AssignSoldier:registerAssetId("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_1_Ladder" .. _ladderNumberBase .. "_Tiling1Part", "ASSIGN_SOLDIER_LADDER" .. _ladderNumberBase .. "_TILING1_PREFAB")
AssignSoldier:registerAssetId("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_2_Ladder" .. _ladderNumberBase .. "_Tiling2Part", "ASSIGN_SOLDIER_LADDER" .. _ladderNumberBase .. "_TILING2_PREFAB")
--------------------------------------------------------------------------

local defaultNodePrefixList = {
	"ASSIGN_SOLDIER_LADDER" .. _ladderNumberBase .. "_TILING1",
	"ASSIGN_SOLDIER_LADDER" .. _ladderNumberBase .. "_TILING2"
}

-- Register simple building part assets
for i, nodePrefix in ipairs(defaultNodePrefixList) do
	registerDefaultBuildingPart(nodePrefix)
end

-- Register the base root part
AssignSoldier:register({
	DataType = "BUILDING_PART",
	Id = "ASSIGN_SOLDIER_LADDER" .. _ladderNumberBase .. "_ROOT_PART",
	Name = "ASSIGN_SOLDIER_LADDER" .. _ladderNumberBase .. "_ROOT_NAME",
	Description = "ASSIGN_SOLDIER_LADDER" .. _ladderNumberBase .. "_ROOT_DESC",
	Category = "EXTENSION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = "ASSIGN_SOLDIER_LADDER" .. _ladderNumberBase .. "_ROOT_PREFAB",
		EndPart = "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_ROOT_PART", 
		FillerList = {
			"ASSIGN_SOLDIER_LADDER" .. _ladderNumberBase .. "_TILING2_PART"
                },
		BasementFillerList = {
			"ASSIGN_SOLDIER_LADDER" .. _ladderNumberBase .. "_TILING1_PART"
         	},
		MinimumScale = 0,
		BasementScale = 1,
		IsVertical = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = _resourceNeeded, Quantity = 0 }
		}
	}
})

--------------------------------------------------------------------------

-- Register flag assigner component on the collider's type to 'PLATFORM'

--AssignSoldier:configurePrefabFlagList("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_3_Ladder" .. _ladderNumberBase .. "_TopPart", { "PLATFORM" })
AssignSoldier:configurePrefabFlagList("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_2_Ladder" .. _ladderNumberBase .. "_Tiling2Part", { "PLATFORM" })
AssignSoldier:configurePrefabFlagList("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_1_Ladder" .. _ladderNumberBase .. "_Tiling1Part", { "PLATFORM" })
 
-- Override building part components properties (generated by the building asset processor) to add building zones
AssignSoldier:registerPrefabComponent("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_0_Ladder" .. _ladderNumberBase .. "_RootPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 2, 4 }
})
end

registerSpecialLadder("13", "14", "WOOD")

function registerBasicSimpleLadder(_ladderNumber)
-- Apply building asset processor
AssignSoldier:registerAssetProcessor("models/basic/basicLadder" .. _ladderNumber .. ".fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register Main prefab nodes
AssignSoldier:registerAssetId("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_0_Ladder" .. _ladderNumber .. "_RootPart", "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_ROOT_PREFAB")

-- Register the stone gate set small squared tower root part
AssignSoldier:register({
	DataType = "BUILDING_PART",
	Id = "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_ROOT_PART",
	Name = "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_ROOT_NAME",
	Description = "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_ROOT_DESC",
	Category = "EXTENSION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "ASSIGN_SOLDIER_LADDER" .. _ladderNumber .. "_ROOT_PREFAB"
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 0 }
		}
	},
	AssetBuildingFunction = "ASSIGN_SOLDIER_FUNCTION_LADDER" .. _ladderNumber
})
AssignSoldier:register({
		DataType = "BUILDING_FUNCTION_WORKPLACE",
                Id = "ASSIGN_SOLDIER_FUNCTION_LADDER" .. _ladderNumber,
		WorkerCapacity = 1,
                RelatedJob = { Job = "SOLDIER", Behavior = "WORK_BEHAVIOR" }
})

-- Override building part components properties (generated by the building asset processor) to add building zones
AssignSoldier:registerPrefabComponent("models/basic/basicLadder" .. _ladderNumber .. ".fbx/Prefab/ladder/step_0_Ladder" .. _ladderNumber .. "_RootPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 0, 0 }
})
end

registerBasicSimpleLadder("5")

